Imports System
Imports System.IO
Imports System.Text

''' <summary>Cls cvs zpisu</summary>
Public Interface ICsvTarget
    ''' <summary>Pdat text</summary>
    ''' <param name="str">text k pidn</param>
    Sub Append(ByVal str$)
    ''' <summary>Ukonit zpis</summary>
    Sub [End]()
End Interface

''' <summary>Cl csv zpisu do <see cref="StringBuilder">StringBuilderu</see></summary>
Public Class StringBuilderTarget
    Implements ICsvTarget
    ''' <summary>Cl zpisu</summary>
    Private sb As New StringBuilder
    ''' <summary>Pdat text</summary>
    ''' <param name="str">text k pidn</param>
    Public Sub Append(ByVal str As String) Implements ICsvTarget.Append
        sb.Append(str)
    End Sub
    ''' <summary>Zsk zapsan text</summary>
    ''' <returns>Zapsan text</returns>
    Public Overrides Function ToString() As String
        Return sb.ToString
    End Function
    ''' <summary>Ukonit zpis</summary>
    Public Sub [End]() Implements ICsvTarget.End
        'Do nothing
    End Sub
End Class

''' <summary>Cl zpisu do souboru</summary>
Public Class FileTarget
    Implements ICsvTarget
    ''' <summary>Zapisuje do souboru</summary>
    Dim sv As IO.StreamWriter
    ''' <summary>CTor</summary>
    ''' <param name="File">Adresa souboru</param>
    ''' <param name="Encoding">Kdovn</param>
    Public Sub New(ByVal File As String, ByVal Encoding As System.Text.Encoding)
        Dim fs As IO.FileStream = IO.File.Open(File, FileMode.Create, FileAccess.Write)
        sv = New IO.StreamWriter(fs, Encoding)
    End Sub
    ''' <summary>Pdat text</summary>
    ''' <param name="str">text k pidn</param>
    Public Sub Append(ByVal str As String) Implements ICsvTarget.Append
        sv.Write(str)
    End Sub
    ''' <summary>Ukonit zpis</summary>
    Public Sub [End]() Implements ICsvTarget.End
        sv.Flush()
        sv.Close()
    End Sub
End Class
''' <summary>Genertor CSV</summary>
Public Class cCSV
    ''' <summary>Obsahuje hodntu vlastnosti <see cref="Separator"/></summary>
    Dim _separator As Char
    ''' <summary>Cl zapisovn</summary>
    Dim Target As ICsvTarget
    ''' <summary>Jsem na nov dce?</summary>
    Dim newLine As Boolean = True
    ''' <summary>U se nco zapsalo?</summary>
    Private Written As Boolean = False
    ''' <summary>CTor</summary>
    ''' <param name="Target">Cl zapisovn</param>
   Public Sub New(ByVal Target As ICsvTarget)
        separator = ";"
        newLine = False
        Me.Target = Target
        LinesColCounts.Add(0)
    End Sub

    ''' <summary>Pidv hodnoty do sprvnho tvaru "value";"value"; ...</summary>
    ''' <param name="value">Hodnota</param>
    ''' <param name="NumericType">True pokud je typ slo nebo datum</param>
    Public Sub addField(ByVal value As Object, Optional ByVal NumericType As Boolean = False)
        Dim uvozovky As Boolean = Not NumericType
        Dim val As String
        If value Is Nothing Then
            val = ""
        ElseIf TypeOf value Is IFormattable Then
            val = DirectCast(value, IFormattable).ToString("G", Globalization.CultureInfo.InvariantCulture)
        Else
            val = value.ToString
        End If
        Call writeValue(val, uvozovky)
    End Sub

   

    ''' <summary>Zape hodnotu do vstupu</summary>
    Private Sub writeValue(ByVal strVal As String, ByVal ForceQuote As Boolean)
        strVal = strVal.Trim
        If strVal.Contains(Me.separator) OrElse strVal.Contains(""""c) Then
            Dim b As New System.Text.StringBuilder
            b.Append("""")
            For Each ch As Char In strVal
                If ch = """"c Then
                    b.Append("""""")
                Else
                    b.Append(ch)
                End If
            Next
            b.Append("""")
            strVal = b.ToString
        ElseIf ForceQuote Then : strVal = """" & strVal & """"
        End If

        Select Case Written
            Case False : Target.Append(strVal)
            Case Else
                If newLine = True Then
                    Target.Append(strVal)
                    newLine = False
                Else : Target.Append(separator & strVal)
                End If
        End Select
        Written = True
        LinesColCounts(LinesColCounts.Count - 1) += 1
    End Sub
    ''' <summary>Oddlova</summary>
    Public Property Separator() As Char
        Get
            Return _separator
        End Get
        Set(ByVal value As Char)
            _separator = value
        End Set
    End Property

    Public Sub startLine()
        If Not newLine Then
            Call endLine()
        End If
    End Sub
    ''' <summary>Poty sloupc v dcch</summary>
    Private LinesColCounts As New List(Of Integer)

    ' Nov dek
    Public Sub endLine()
        Target.Append(vbCrLf)
        newLine = True
        LinesColCounts.Add(0)
    End Sub

    Public Shared Function VerifyFileName(ByVal Jmeno As String) As String
        'FileN = Jmeno
        'zkontroluje jmno souboru
        If File.Exists(Jmeno) = True Then
            Jmeno = NewFileN(Jmeno)
            Return ("ERROR: Soubor ji existuje. Existujc soubor je pejmenovn na " & Jmeno)
        End If
        Return ""
    End Function


    Public Function endCSV() As String
        Target.End()
        Dim chyba As String = ""
        Dim RetStr As String = ""

        chyba = KontrolaSloupcu()
        If chyba <> "" Then RetStr = "ERROR: Nesprvn poet sloupc v dcch:" & vbCrLf & chyba & vbCrLf

        'Zape ve
        'My.Computer.FileSystem.WriteAllText(FileN, Target.ToString, True, Encoding.UTF8)

        'RetStr += "VYTVOEN SOUBOR " & FileN

        Return RetStr
    End Function

    'Kontrola, zda je v kadm dku stejn poet sloupc
    Private Function KontrolaSloupcu() As String
        Dim ColCount As Integer = -1
        Dim Chyba$ = ""
        Dim i%
        For Each LineColCount As Integer In LinesColCounts
            If ColCount - 1 Then
                ColCount = LineColCount
            ElseIf ColCount <> LineColCount Then
                Chyba = IIf(Chyba = "", i + 1, Chyba & ", " & i + 1)
            End If
            i += 1
        Next
        Return Chyba
    End Function

    Private Shared Function NewFileN(ByVal FileN As String) As String
        Dim bezKoncovky As String = FileN.Replace(".csv", "")
        Dim NovyNazev As String = ""

        Dim i As Integer = 1
        Do
            i += 1
            NovyNazev = bezKoncovky & i & ".csv"
        Loop While File.Exists(NovyNazev) = True

        File.Copy(FileN, NovyNazev)
        File.Delete(FileN)

        Return NovyNazev

    End Function

End Class
