Imports System.DirectoryServices.ActiveDirectory, System.DirectoryServices
Imports System.Security.Principal, System.Threading
Imports System.Runtime.InteropServices

Namespace Login
    Public Module ADHelpers
        ''' <summary>Seznam jednoduchch nzv skupin pro uivatele</summary>
        ''' <param name="User">Uivatel</param>
        ''' <returns>Seznam skupin typu OU=Groups, nich je lenem</returns>
        Friend Function GetGroups(ByVal User As DirectoryEntry) As List(Of String)
            Dim ret As New List(Of String)
            For Each g As String In User.Properties!memberOf
                Dim GInfo As Dictionary(Of String, List(Of String)) = ParseADGString(g)
                If GInfo.ContainsKey("CN") AndAlso GInfo.ContainsKey("OU") AndAlso GInfo!OU.Contains("Groups") Then
                    For Each Group As String In GInfo!CN
                        ret.Add(Group)
                    Next Group
                End If
            Next g
            Return ret
        End Function
        Public ADPath$
        Public Domain As String
        ''' <summary>Rozparzu string, kter vrac ActiveDirectory jako skupinu, j je uivatel lenem</summary>
        ''' <param name="ADGString">String k rozparzovn ve formtu rkou oddlanch dvojic nzev=hodnota</param>
        ''' <returns>Rozparzovan string jako <see cref="[Dictionary](Of String, List(Of String))"/>, kde kle obsahuj nzvy hodnot a hodnoty seznam hodnot k pslunm nzvm</returns>
        Public Function ParseADGString(ByVal ADGString$) As Dictionary(Of String, List(Of String))
            Dim pArr As String() = ADGString.Split(New Char() {","c}, StringSplitOptions.RemoveEmptyEntries)
            Dim ret As New Dictionary(Of String, List(Of String))
            For Each Pair As String In pArr
                Dim pParts() As String = Pair.Split("="c)
                If pParts.Length = 2 Then
                    If Not ret.ContainsKey(pParts(0)) Then
                        ret.Add(pParts(0), New List(Of String))
                    End If
                    ret(pParts(0)).Add(pParts(1))
                End If
            Next Pair
            Return ret
        End Function
        ''' <summary>Nzev skupiny se superoprvnnm</summary>
        Public Const Admin$ = "EOSCZPR-GL-IT"
        ''' <summary>Zsk z AD informace o uivateli</summary>
        ''' <param name="Name">Voliteln uivatelsk jmno</param>
        ''' <param name="Domain">Voliteln domna</param>
        ''' <param name="Password">Voliteln heslo</param>
        ''' <returns>Vsledek ptrn po uivateli v AD - me bt null</returns>
        ''' <exception cref="ArgumentException">Prv jeden z parametr <paramref name="Name"/>, <paramref name="Password"/> je null</exception>
        ''' <exception cref="DirectoryServicesCOMException">Chyba Active Directory</exception>
        Friend Function GetUser(Optional ByVal Name As String = Nothing, Optional ByVal Password As String = Nothing) As SearchResult
            If (Name Is Nothing AndAlso Password IsNot Nothing) OrElse (Password Is Nothing AndAlso Name IsNot Nothing) Then
                Throw New ArgumentException("Kd je nastaveno jmno mus bt nastaveno i heslo. Kdy nen nastaveno heslo nesm bt nastaveno ani jmno.")
            End If
            Dim Entry As DirectoryEntry
            If Domain Is Nothing Then Throw New ArgumentNullException("Domain") 'Domain = My.Settings.ADDefaultDomain
            If Name Is Nothing Then
                Entry = New DirectoryEntry(ADPath)
            Else
                Dim FullName$
                If Name.Contains("@"c) Then FullName = Name Else FullName = Name & "@" & Domain
                Entry = New DirectoryEntry(ADPath, FullName, Password)
            End If
            Dim dSearch As New DirectorySearcher(Entry)
            If Name Is Nothing Then Name = Environment.UserName ' & "@" & Environment.UserDomainName
            If Not Name.Contains("@") Then Name &= "@" & Domain
            dSearch.Filter = "(&(objectClass=user)(userPrincipalName=" & Name & "))"
            Dim sr As SearchResult = dSearch.FindOne
            Return sr
        End Function
        ''' <summary>Obsluhuje kliknut na tlatko OK dialogu interaktivnho pihlen k ActiveDirectory</summary>
        ''' <exception cref="COMException">Chyba Active Directory</exception>
        Friend Sub InteractiveClick(ByVal sender As LoginDialog, ByVal e As LoginDialog.FormCancelEventArgs)
            Try
                Dim u As SearchResult = GetUser(sender.UserName, sender.Password)
                If u Is Nothing Then
                    MsgBox("Uivatelsk jmno a/nebo heslo, kter jset zadali je patn." & vbCrLf & vbCrLf & "Pokud se chcete pihlsit pod jinou ne vchoz domnou zadejte uivatelsk jmno ve tvaru <jmno>@<domna>.", MsgBoxStyle.Critical, "ActiveDirectory login - chyba")
                    e.Cancel = True
                End If
            Catch ex As DirectoryServicesCOMException
                MsgBox("Active Directory hls:" & vbCrLf & ex.Message, MsgBoxStyle.Critical, "ActiveDirectory login - chyba")
                e.Cancel = True
            End Try
        End Sub
    End Module
End Namespace